diff --git a/Htmldocs/release_notes.html b/Htmldocs/release_notes.html
index bedbe1fa..36b36660 100644
--- a/Htmldocs/release_notes.html
+++ b/Htmldocs/release_notes.html
@@ -16,7 +16,7 @@
The OSCAR Team
- [new] Add preliminary support for ResMed AirSense 11 CPAP machines.
- - [new] Add support for Fisher & Paykel SleepStyle CPAP machines.
+ - [new] Add support for Fisher & Paykel SleepStyle CPAP machines.
- [new] Add support for DeVilbiss BLUE (DV6x) CPAP machines.
- [new] Additional Philips Respironics devices tested and fully supported:
diff --git a/oscar/SleepLib/loader_plugins/sleepstyle_loader.h b/oscar/SleepLib/loader_plugins/sleepstyle_loader.h
index cc576e73..58e1daa0 100644
--- a/oscar/SleepLib/loader_plugins/sleepstyle_loader.h
+++ b/oscar/SleepLib/loader_plugins/sleepstyle_loader.h
@@ -38,7 +38,7 @@ class SleepStyle: public CPAP
const int sleepstyle_load_buffer_size = 1024 * 1024;
-extern ChannelID SS_SenseAwakeLevel, SS_EPR, SS_EPRLevel, SS_Ramp, SS_Humidity;
+extern ChannelID SS_SensAwakeLevel, SS_EPR, SS_EPRLevel, SS_Ramp, SS_Humidity;
const QString sleepstyle_class_name = STR_MACH_SleepStyle;
diff --git a/oscar/SleepLib/loader_plugins/somnopose_loader.cpp b/oscar/SleepLib/loader_plugins/somnopose_loader.cpp
index 9f410d04..1def0b8b 100644
--- a/oscar/SleepLib/loader_plugins/somnopose_loader.cpp
+++ b/oscar/SleepLib/loader_plugins/somnopose_loader.cpp
@@ -94,7 +94,7 @@ int SomnoposeLoader::OpenFile(const QString & filename)
}
QDateTime epoch(QDate(2001, 1, 1));
- qint64 ep = qint64(epoch.toTime_t()+epoch.offsetFromUtc()) * 1000, time;
+ qint64 ep = qint64(epoch.toTime_t()+epoch.offsetFromUtc()) * 1000, time=0;
qDebug() << "Epoch starts at" << epoch.toString();
double timestamp, orientation=0, inclination=0, movement=0;
@@ -102,7 +102,8 @@ int SomnoposeLoader::OpenFile(const QString & filename)
QStringList fields;
bool ok, orientation_ok, inclination_ok, movement_ok;
- bool first = true;
+ bool first = true, skip_session = false;
+ int session_count = 0;
MachineInfo info = newInfo();
info.model = model;
info.serial = serial;
@@ -115,6 +116,39 @@ int SomnoposeLoader::OpenFile(const QString & filename)
while (!(data = ts.readLine()).isEmpty()) {
fields = data.split(",");
+ if (fields.size() >= col_timestamp && fields[col_timestamp] == "-") {
+ // Flag end of session...
+ if (sess) {
+ if (ev_orientation) {
+ sess->setMin(POS_Orientation, ev_orientation->Min());
+ sess->setMax(POS_Orientation, ev_orientation->Max());
+ }
+ if (ev_inclination) {
+ sess->setMin(POS_Inclination, ev_inclination->Min());
+ sess->setMax(POS_Inclination, ev_inclination->Max());
+ }
+ if (ev_movement) {
+ sess->setMin(POS_Movement, ev_movement->Min());
+ sess->setMax(POS_Movement, ev_movement->Max());
+ }
+ sess->really_set_last(time);
+ sess->SetChanged(true);
+
+ mach->AddSession(sess);
+ session_count++;
+ }
+ // Prepare for potential next session...
+ sess = nullptr;
+ ev_orientation = ev_inclination = ev_movement = nullptr;
+ first = true;
+ skip_session = false;
+ continue;
+ }
+
+ if (skip_session) {
+ continue;
+ }
+
if (fields.size() < hdr_size) { // missing fields.. skip this record
continue;
}
@@ -122,6 +156,7 @@ int SomnoposeLoader::OpenFile(const QString & filename)
timestamp = fields[col_timestamp].toDouble(&ok);
if (!ok) { continue; }
+ orientation_ok = inclination_ok = movement_ok = false;
if (col_orientation >= 0) {
orientation = fields[col_orientation].toDouble(&orientation_ok);
@@ -142,12 +177,15 @@ int SomnoposeLoader::OpenFile(const QString & filename)
time = (timestamp * 1000.0) + ep;
if (first) {
+ first = false;
sid = time / 1000;
qDebug() << "First timestamp is" << QDateTime::fromMSecsSinceEpoch(time).toString();
if (mach->SessionExists(sid)) {
- qDebug() << "File " << filename << " already loaded... skipping";
- return 0; // Already imported
+ qDebug() << "File " << filename << " session " << sid << " already loaded... skipping";
+ // Continue processing file to allow for case where new sessions are added to a file
+ skip_session = true;
+ continue;
}
sess = new Session(mach, sid);
@@ -161,7 +199,6 @@ int SomnoposeLoader::OpenFile(const QString & filename)
if (col_movement >= 0) {
ev_movement = sess->AddEventList(POS_Movement, EVL_Event, 1, 0, 0, 0);
}
- first = false;
}
sess->set_last(time);
@@ -193,14 +230,17 @@ int SomnoposeLoader::OpenFile(const QString & filename)
sess->SetChanged(true);
mach->AddSession(sess);
+ session_count++;
+ }
+ if (session_count) {
mach->Save();
// Adding these to hopefully make data persistent...
mach->SaveSummaryCache();
p_profile->StoreMachines();
}
- return 1;
+ return session_count;
}
diff --git a/oscar/SleepLib/loader_plugins/sleepstyle_loader.cpp b/oscar/SleepLib/loader_plugins/sleepstyle_loader.cpp
index 3c641498..f18b1879 100644
--- a/oscar/SleepLib/loader_plugins/sleepstyle_loader.cpp
+++ b/oscar/SleepLib/loader_plugins/sleepstyle_loader.cpp
@@ -21,7 +21,7 @@
const QString FPHCARE = "FPHCARE";
-ChannelID SS_SenseAwakeLevel;
+ChannelID SS_SensAwakeLevel;
ChannelID SS_EPR;
ChannelID SS_EPRLevel;
ChannelID SS_Ramp;
@@ -276,7 +276,7 @@ int SleepStyleLoader::OpenMachine(Machine *mach, const QString & path, const QSt
SessionID sid;//,st;
float hours, mins;
- Q_UNUSED(mins) // Used only in debug mode
+
// For diagnostics, print summary of last 20 session or one week
qDebug() << "SS Loader - last 20 Sessions:";
@@ -300,9 +300,7 @@ int SleepStyleLoader::OpenMachine(Machine *mach, const QString & path, const QSt
hours = sess->hours();
mins = hours * 60;
dt = QDateTime::fromTime_t(sid);
-#ifdef DEBUGSS
qDebug() << cnt << ":" << dt << "session" << sid << "," << mins << "minutes" << a;
-#endif
if (dt.date() < date) {
break;
}
@@ -383,7 +381,7 @@ quint32 ssconvertDate(quint32 timestamp)
QDateTime dt = QDateTime(QDate(year, month, day), QTime(hour, minute, second), Qt::UTC);
#ifdef DEBUGSS
- qDebug().noquote() << "SS timestamp" << timestamp << year << month << day << dt << hour << minute << second;
+// qDebug().noquote() << "SS timestamp" << timestamp << year << month << day << dt << hour << minute << second;
#endif
// Q NO!!! _ASSERT(dt.isValid());
@@ -585,16 +583,16 @@ bool SleepStyleLoader::OpenSummary(Machine *mach, const QString & filename)
quint32 ts;
//QByteArray line;
- unsigned char ramp, p3, j1, x1, mode;
+ unsigned char ramp, j1, x1, x2, mode;
unsigned char runTime, useTime, minPressSet, maxPressSet, minPressSeen, pct95PressSeen, maxPressSeen;
- unsigned char senseAwakeLevel, humidityLevel, EPRLevel;
+ unsigned char sensAwakeLevel, humidityLevel, EPRLevel;
unsigned char CPAPpressSet, flags;
quint16 c1, c2, c3, c4;
// quint16 d1, d2, d3;
unsigned char d1, d2, d3, d4, d5, d6;
- int usage; //,runtime;
+ int usage;
QDate date;
@@ -621,7 +619,7 @@ bool SleepStyleLoader::OpenSummary(Machine *mach, const QString & filename)
ts = ssconvertDate(ts);
#ifdef DEBUGSS
- qDebug() << "\nSS SUM Session" << nblock << "with timestamp" << ts << QDateTime::fromSecsSinceEpoch(ts).toString("MM/dd/yyyy hh:mm:ss");
+ qDebug() << "\nSS SUM Session" << nblock << "ts" << ts << QDateTime::fromSecsSinceEpoch(ts).toString("MM/dd/yyyy hh:mm:ss");
#endif
// the following two quite often match in value
in >> runTime; // 0x04
@@ -648,14 +646,14 @@ bool SleepStyleLoader::OpenSummary(Machine *mach, const QString & filename)
in >> mode; // 0x18
in >> ramp; // 0x19
- in >> p3; // 0x1a
+ in >> x1; // 0x1a
- in >> x1; // 0x1b
+ in >> x2; // 0x1b
in >> CPAPpressSet; // 0x1c
in >> minPressSet;
in >> maxPressSet;
- in >> senseAwakeLevel;
+ in >> sensAwakeLevel;
in >> humidityLevel;
in >> EPRLevel;
in >> flags;
@@ -664,20 +662,18 @@ bool SleepStyleLoader::OpenSummary(Machine *mach, const QString & filename)
unsigned char s [5];
for (unsigned int i=0; i < sizeof(s); i++)
in >> s[i];
-#ifdef DEBUGSS
- qDebug() << "SS SUM block" << nblock
- << "a:" <<"Pressure Min"<SessionExists(ts)) {
Session *sess = new Session(mach, ts);
sess->really_set_first(qint64(ts) * 1000L);
@@ -706,9 +702,9 @@ bool SleepStyleLoader::OpenSummary(Machine *mach, const QString & filename)
sess->settings[SS_Ramp] = ramp;
if (flags & 0x04)
- sess->settings[SS_SenseAwakeLevel] = senseAwakeLevel / 10.0;
+ sess->settings[SS_SensAwakeLevel] = sensAwakeLevel / 10.0;
else
- sess->settings[SS_SenseAwakeLevel] = 0;
+ sess->settings[SS_SensAwakeLevel] = 0;
sess->settings[CPAP_PresReliefMode] = PR_EPR;
@@ -874,7 +870,7 @@ bool SleepStyleLoader::OpenDetail(Machine *mach, const QString & filename)
a4 = data[idx + 5]; // [0..5] UF1, [6..7] Unknown
a5 = data[idx + 6]; // [0..5] UF2, [6..7] Unknown
// SenseAwake bits are in the first two bits of the last three data fields
// TODO: Confirm that the bits are in the right order
a6 = (a3 >> 6) << 4 | ((a4 >> 6) << 2) | (a5 >> 6);
@@ -928,23 +924,23 @@ void SleepStyleLoader::initChannels()
using namespace schema;
Channel * chan = nullptr;
- channel.add(GRP_CPAP, chan = new Channel(SS_SenseAwakeLevel = 0xf305, SETTING, MT_CPAP, SESSION,
- "SenseAwakeLevel-ss",
- QObject::tr("SenseAwake level"),
- QObject::tr("SenseAwake level"),
- QObject::tr("SenseAwake"),
+ channel.add(GRP_CPAP, chan = new Channel(SS_SensAwakeLevel = 0xf305, SETTING, MT_CPAP, SESSION,
+ "SensAwakeLevel-ss",
+ QObject::tr("SensAwake level"),
+ QObject::tr("SensAwake level"),
+ QObject::tr("SensAwake"),
STR_UNIT_CMH2O, DEFAULT, Qt::black));
chan->addOption(0, STR_TR_Off);
channel.add(GRP_CPAP, chan = new Channel(SS_EPR = 0xf306, SETTING, MT_CPAP, SESSION,
- "EPR-ss", QObject::tr("EPR"), QObject::tr("Exhale Pressure Relief"), QObject::tr("EPR"),
+ "EPR-ss", QObject::tr("EPR"), QObject::tr("Expiratory Relief"), QObject::tr("EPR"),
"", DEFAULT, Qt::black));
chan->addOption(0, STR_TR_Off);
chan->addOption(1, STR_TR_On);
channel.add(GRP_CPAP, chan = new Channel(SS_EPRLevel = 0xf307, SETTING, MT_CPAP, SESSION,
- "EPRLevel-ss", QObject::tr("EPR Level"), QObject::tr("Exhale Pressure Relief Level"), QObject::tr("EPR Level"),
+ "EPRLevel-ss", QObject::tr("EPR Level"), QObject::tr("Expiratory Relief Level"), QObject::tr("EPR Level"),
STR_UNIT_CMH2O, INTEGER, Qt::black));
chan->addOption(0, STR_TR_Off);
@@ -874,7 +870,7 @@ bool SleepStyleLoader::OpenDetail(Machine *mach, const QString & filename)
a4 = data[idx + 5]; // [0..5] UF1, [6..7] Unknown
a5 = data[idx + 6]; // [0..5] UF2, [6..7] Unknown
- // SenseAwake bits are in the first two bits of the last three data fields
+ // SensAwake bits are in the first two bits of the last three data fields
// TODO: Confirm that the bits are in the right order
a6 = (a3 >> 6) << 4 | ((a4 >> 6) << 2) | (a5 >> 6);